"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const crypto_1 = require("crypto");
const jsonrpc2_ws_1 = require("jsonrpc2-ws");
const rxjs_1 = require("rxjs");
const operators_1 = require("rxjs/operators");
class RealtimeClient {
    constructor() {
        this.client = new jsonrpc2_ws_1.Client('wss://ws.lightstream.bitflyer.com/json-rpc');
        this.message$ = new rxjs_1.Observable((subscriber) => {
            this.client.methods.set('channelMessage', (_, params) => {
                subscriber.next(params);
            });
        });
    }
    unsubscribe() {
        this.client.disconnect();
    }
    board(product) {
        return this.subscribe(`lightning_board_${product}`);
    }
    boardSnapshot(product) {
        return this.subscribe(`lightning_board_snapshot_${product}`);
    }
    executions(product) {
        return this.subscribe(`lightning_executions_${product}`).pipe(operators_1.mergeAll());
    }
    ticker(product) {
        return this.subscribe(`lightning_ticker_${product}`);
    }
    subscribe(channel) {
        this.client.call('subscribe', { channel });
        return this.message$.pipe(operators_1.filter((params) => params.channel === channel), operators_1.map(({ message }) => message));
    }
    auth(key, secret) {
        const timestamp = Date.now();
        const nonce = crypto_1.randomBytes(16).toString('hex');
        const signature = crypto_1.createHmac('sha256', secret)
            .update(timestamp + nonce).digest('hex');
        return this.client.call('auth', {
            api_key: key, timestamp, nonce, signature,
        });
    }
}
exports.default = RealtimeClient;
